@@ -2,6 +2,7 @@
 
 #include <config.h>
 #include "trans.h"
+#include "xmlstar.h"
 
 /*
  *  This code is based on xsltproc by Daniel Veillard (daniel@veillard.com)
@@ -12,7 +13,6 @@
 xmlChar *paths[MAX_PATHS + 1];
 int nbpaths = 0;
 
-const char *output = NULL;          /* file name to save output */
 int errorno = 0;
 
 xmlExternalEntityLoader defaultEntityLoader = NULL;
@@ -192,15 +192,13 @@ xsltExternalEntityLoader(const char *URL, const char *ID, xmlParserCtxtPtr ctxt)
     return(NULL);
 }
 
-/**
- *  Run stylesheet on XML document
- */
-void
-xsltProcess(xsltOptionsPtr ops, xmlDocPtr doc, const char** params,
+/* get result of XSL transformation */
+xmlDocPtr
+xsltTransform(xsltOptionsPtr ops, xmlDocPtr doc, const char** params,
             xsltStylesheetPtr cur, const char *filename)
 {
-    xmlDocPtr res;
     xsltTransformContextPtr ctxt;
+    xmlDocPtr res;
 
     if (ops->omit_decl)
     {
@@ -210,60 +208,40 @@ xsltProcess(xsltOptionsPtr ops, xmlDocPtr doc, const char** params,
 #ifdef LIBXML_XINCLUDE_ENABLED
     if (ops->xinclude) xmlXIncludeProcess(doc);
 #endif
-    if (output == NULL)
-    {
-        ctxt = xsltNewTransformContext(cur, doc);
-        if (ctxt == NULL) return;
 
-        res = xsltApplyStylesheetUser(cur, doc, params, NULL, NULL, ctxt);
+    ctxt = xsltNewTransformContext(cur, doc);
+    if (ctxt == NULL) return NULL;
+
+    res = xsltApplyStylesheetUser(cur, doc, params, NULL, NULL, ctxt);
         
-        if (ctxt->state == XSLT_STATE_ERROR)
-            errorno = 9;
-        if (ctxt->state == XSLT_STATE_STOPPED)
-            errorno = 10;
-        xsltFreeTransformContext(ctxt);
-        xmlFreeDoc(doc);
-        if (res == NULL)
-        {
-            fprintf(stderr, "no result for %s\n", filename);
-            return;
-        }
-
-        if (cur->methodURI == NULL)
-        {
-            xsltSaveResultToFile(stdout, res, cur);
-        }
-        else
-        {
-            if (xmlStrEqual(cur->method, (const xmlChar *) "xhtml"))
-            {
-                fprintf(stderr, "non standard output xhtml\n");
-                xsltSaveResultToFile(stdout, res, cur);
-            }
-            else
-            {
-                fprintf(stderr, "unsupported non standard output %s\n",
-                        cur->method);
-                errorno = 7;
-            }
-        }
-
-        xmlFreeDoc(res);
-    }
-    else
+    if (ctxt->state == XSLT_STATE_ERROR)
+        errorno = 9;
+    if (ctxt->state == XSLT_STATE_STOPPED)
+        errorno = 10;
+    xsltFreeTransformContext(ctxt);
+    xmlFreeDoc(doc);
+    if (res == NULL)
     {
-        int ret;
-
-        ctxt = xsltNewTransformContext(cur, doc);
-        if (ctxt == NULL) return;
-
-        ret = xsltRunStylesheet(cur, doc, params, output, NULL, NULL);
-
-        if (ctxt->state == XSLT_STATE_ERROR) errorno = 9;
-
-        xsltFreeTransformContext(ctxt);
-        xmlFreeDoc(doc);
+        fprintf(stderr, "no result for %s\n", filename);
     }
+    return res;
+}
+
+/**
+ *  Run stylesheet on XML document
+ */
+void
+xsltProcess(xsltOptionsPtr ops, xmlDocPtr doc, const char** params,
+            xsltStylesheetPtr cur, const char *filename)
+{
+    xmlDocPtr res = xsltTransform(ops, doc, params, cur, filename);
+
+    if (res && xsltSaveResultToFile(stdout, res, cur) < 0)
+    {
+        errorno = EXIT_LIB_ERROR;
+    }
+
+    xmlFreeDoc(res);
 }
 
 /**
