@@ -91,8 +91,10 @@ typedef struct _selOptions {
 typedef selOptions *selOptionsPtr;
 
 typedef enum { TARG_NONE = 0, TARG_SORT_OP, TARG_XPATH,
-               TARG_ATTR_STRING, TARG_STRING,
-               TARG_NEWLINE, TARG_NO_CMDLINE = TARG_NEWLINE, TARG_INP_NAME
+               TARG_ATTR_STRING, TARG_STRING, TARG_VAR,
+               /* template args below don't consume any command line args */
+               TARG_NEWLINE, TARG_NO_CMDLINE = TARG_NEWLINE,
+               TARG_INP_NAME, TARG_STR_NAME_SELECT
 } template_argument_type;
 typedef struct {
     const xmlChar *attrname;
@@ -147,7 +149,9 @@ static const char select_usage_str_4[] =
 "  -o or --output <string>   - output string literal\n"
 "  -n or --nl                - print new line\n"
 "  -f or --inp-name          - print input file name (or URL)\n"
-"  -m or --match <xpath>     - match XPATH expression\n";
+"  -m or --match <xpath>     - match XPATH expression\n"
+"  --var <name> <value> --break or\n"
+"  --var <name>=<value>      - declare a variable (referenced by $name)\n";
 
 static const char select_usage_str_5[] =
 "  -i or --if <test-xpath>   - check condition <xsl:if test=\"test-xpath\">\n"
@@ -170,7 +174,8 @@ static const char select_usage_str_6[] =
 static const template_option
     OPT_TEMPLATE = { 't', "template" },
     OPT_COPY_OF  = { 'c', "copy-of", BAD_CAST "copy-of", {{BAD_CAST "select", TARG_XPATH}}, 0 },
-    OPT_VALUE_OF = { 'v', "value-of", BAD_CAST "value-of", {{BAD_CAST "select", TARG_XPATH}}, 0 },
+    OPT_VALUE_OF = { 'v', "value-of", BAD_CAST "with-param", {{BAD_CAST "name", TARG_STR_NAME_SELECT},
+                                                              {BAD_CAST "select", TARG_XPATH}}, -1 },
     OPT_OUTPUT   = { 'o', "output", BAD_CAST "text", {{NULL, TARG_STRING}}, 0 },
     OPT_NL       = { 'n', "nl", BAD_CAST "value-of", {{NULL, TARG_NEWLINE}}, 0 },
     OPT_INP_NAME = { 'f', "inp-name", BAD_CAST "copy-of", {{NULL, TARG_INP_NAME}}, 0 },
@@ -182,6 +187,7 @@ static const template_option
     OPT_ATTR     = { 'a', "attr", BAD_CAST "attribute", {{BAD_CAST "name", TARG_ATTR_STRING}}, 1 },
     OPT_BREAK    = { 'b', "break", NULL, {{NULL}}, -1 },
     OPT_SORT     = { 's', "sort", BAD_CAST "sort", {{NULL, TARG_SORT_OP}, {BAD_CAST "select", TARG_XPATH}}, 0 },
+    OPT_VAR      = { 0,   "var", BAD_CAST "variable", {{BAD_CAST "name", TARG_VAR}}, 1},
 
     *TEMPLATE_OPTIONS[] = {
         &OPT_TEMPLATE,
@@ -197,7 +203,8 @@ static const template_option
         &OPT_ELEM,
         &OPT_ATTR,
         &OPT_BREAK,
-        &OPT_SORT
+        &OPT_SORT,
+        &OPT_VAR
     };
 
 void
@@ -390,8 +397,8 @@ checkNsRefs(xmlNodePtr root, const char *xpath)
  */
 int
 selGenTemplate(xmlNodePtr root, xmlNodePtr template_node,
-    xmlNsPtr xslns, selOptionsPtr ops, int* use_inputfile, int* lastTempl,
-    int start, int argc, char **argv)
+    xmlNsPtr xslns, selOptionsPtr ops, int* use_inputfile, int* use_value_of,
+    int* lastTempl, int start, int argc, char **argv)
 {
     int i;
     int templateEmpty;
@@ -416,6 +423,7 @@ selGenTemplate(xmlNodePtr root, xmlNodePtr template_node,
         xmlNodePtr newnode = NULL;
         const template_option *newtarg = NULL;
         int j;
+        int nesting;
 
         if (argv[i][0] == '-' && argv[i][1] != '\0')
         {
@@ -460,9 +468,19 @@ selGenTemplate(xmlNodePtr root, xmlNodePtr template_node,
                 exit(EXIT_BAD_ARGS);
             }
         }
+        else if (newtarg == &OPT_VALUE_OF)
+        {
+            node = xmlNewChild(node, xslns, BAD_CAST "call-template", NULL);
+            xmlNewProp(node, BAD_CAST "name", BAD_CAST "value-of-template");
+            node->_private = (void*) &OPT_VALUE_OF;
+            *use_value_of = 1;
+            /* value-of-template uses exslt:node-set */
+            checkNsRefs(root, "exslt:node-set");
+        }
 
         i++;
         templateEmpty = 0;
+        nesting = newtarg->nest;
 
         if (newtarg->xslname)
             newnode = xmlNewChild(node, xslns, newtarg->xslname, NULL);
@@ -473,6 +491,17 @@ selGenTemplate(xmlNodePtr root, xmlNodePtr template_node,
                 selUsage(argv[0], EXIT_BAD_ARGS);
             switch (newtarg->arguments[j].type)
             {
+            case TARG_VAR: {
+                char *equals = strchr(argv[i], '=');
+                if (equals) {
+                    *equals = '\0';
+                    xmlNewProp(newnode, BAD_CAST "select", BAD_CAST (&equals[1]));
+                    nesting = 0;
+                }
+                xmlNewProp(newnode, newtarg->arguments[j].attrname, BAD_CAST argv[i]);
+                break;
+            }
+
             case TARG_XPATH:
                 checkNsRefs(root, argv[i]);
             case TARG_ATTR_STRING:
@@ -486,6 +515,9 @@ selGenTemplate(xmlNodePtr root, xmlNodePtr template_node,
             case TARG_NEWLINE:
                 xmlNewProp(newnode, BAD_CAST "select", BAD_CAST "'\n'");
                 break;
+            case TARG_STR_NAME_SELECT:
+                xmlNewProp(newnode, BAD_CAST "name", BAD_CAST "select");
+                break;
 
             case TARG_INP_NAME:
                 *use_inputfile = 1;
@@ -515,7 +547,7 @@ selGenTemplate(xmlNodePtr root, xmlNodePtr template_node,
             if (newtarg->arguments[j].type < TARG_NO_CMDLINE) i++;
         }
 
-        switch (newtarg->nest) {
+        switch (nesting) {
         case -1:
             do { node = node->parent; } while(node->_private);
             break;
@@ -558,7 +590,7 @@ int
 selPrepareXslt(xmlDocPtr style, selOptionsPtr ops, xmlChar *ns_arr[],
                int start, int argc, char **argv)
 {
-    int i, t, ns, use_inputfile = 0;
+    int i, t, ns, use_inputfile = 0, use_value_of = 0;
     xmlNodePtr root, root_template = NULL;
     xmlNsPtr xslns;
     xmlBufferPtr attr_buf;
@@ -629,7 +661,8 @@ selPrepareXslt(xmlDocPtr style, selOptionsPtr ops, xmlChar *ns_arr[],
             }
 
             i = selGenTemplate(root, template,
-                xslns, ops, &use_inputfile, &lastTempl, i, argc, argv);
+                xslns, ops, &use_inputfile, &use_value_of,
+                &lastTempl, i, argc, argv);
             if (lastTempl) break;
         }
     }
@@ -666,6 +699,22 @@ selPrepareXslt(xmlDocPtr style, selOptionsPtr ops, xmlChar *ns_arr[],
         xmlNewProp(param, BAD_CAST "name", BAD_CAST "inputFile");
     }
 
+    if (use_value_of) {
+        xmlNodePtr value_of_template, for_each, value_of, param;
+        value_of_template = xmlNewChild(root, xslns, BAD_CAST "template", NULL);
+        xmlNewProp(value_of_template, BAD_CAST "name", BAD_CAST "value-of-template");
+        param = xmlNewChild(value_of_template, xslns, BAD_CAST "param", NULL);
+        value_of = xmlNewChild(value_of_template, xslns, BAD_CAST "value-of", NULL);
+        xmlNewProp(value_of, BAD_CAST "select", BAD_CAST "$select");
+        xmlNewProp(param, BAD_CAST "name", BAD_CAST "select");
+        for_each = xmlNewChild(value_of_template, xslns, BAD_CAST "for-each", NULL);
+        xmlNewProp(for_each, BAD_CAST "select", BAD_CAST "exslt:node-set($select)[position()>1]");
+        value_of = xmlNewChild(for_each, xslns, BAD_CAST "value-of", NULL);
+        xmlNewProp(value_of, BAD_CAST "select", BAD_CAST ".");
+        value_of = xmlNewChild(for_each, xslns, BAD_CAST "value-of", NULL);
+        xmlNewProp(value_of, BAD_CAST "select", BAD_CAST "'\n'");
+    }
+
     return i;
 }
 
@@ -713,11 +762,11 @@ do_file(const char *filename, xmlDocPtr style_tree,
         }
 
         res = xsltTransform(xsltOps, doc, params, style, filename);
-        if (!ops->quiet && res && xsltSaveResultToFile(stdout, res, style) < 0)
+        if (!ops->quiet && (!res || xsltSaveResultToFile(stdout, res, style) < 0))
         {
             *status = EXIT_LIB_ERROR;
         }
-        else if ((ops->quiet || *status == EXIT_FAILURE) && res->children)
+        else if ((ops->quiet || *status == EXIT_FAILURE) && res && res->children)
         {
             *status = EXIT_SUCCESS;
             if (ops->quiet) exit(EXIT_SUCCESS);
